%% DC Motor: Open Loop Control


%% Model Parameters

% Nominal ODE Coefficients
a0Nom = 0.94;                      % ODE coefficient, 1/sec
b0Nom = 766.8;                     % ODE coefficient, rad/(sec^2 V)

% Nominal Motor Model
a0 = a0Nom;
b0 = b0Nom;
G = tf(b0,[1 a0]);

% Maximum Operating Voltage, V
umax = 3;

%% Open Loop Controller
% The inputs/outputs of the open loop controller Kol are:
%  Input: Desired rotational speed r, rad/sec
%  Output: Motor voltage u, V
% The open loop controller selects the motor input voltage so that
% the actual motor speed y converges to the desired motor speed r.

% Open Loop Controller
Kol = 1/dcgain(G);

%% Simulations on Nominal Model

% Simulation and plot settings
d = -0.2;        % Input disturbance, V
Tf = 10;         % Final simulation time, sec
co = get(groot,'defaultAxesColorOrder');

% Simulate with several desired velocity inputs
figure(1); clf;
figure(2); clf;
for i=1:2
    % Set numerator & denominator coefficients
    Gnum = b0;       % Numerator coefficients of G
    Gden = [1 a0];   % Denominator coefficients of G

    % Simulate Simulink model
    r = 1000*i;   % Desired speed, rad/sec
    sim('MotorOpenLoop')
    
    % Plot input voltage
    figure(1);
    plot(t,u,'Color',co(i,:));
    hold on;
    
    % Plot motor speed
    figure(2);
    plot(t,y,'Color',co(i,:));
    hold on;
    plot([0 Tf],[r r],'k:');
end

figure(1);
xlabel('Time (sec)');
ylabel('Input u (V)');
grid on;
hold off;
axis([0 Tf 0 umax]);
set(gca, 'LooseInset', get(gca, 'TightInset'));

figure(2);
xlabel('Time (sec)');
ylabel('Speed, y (rad/sec)');
axis([0 Tf 0 2100]);
grid on;
hold off;
set(gca, 'LooseInset', get(gca, 'TightInset'));

if exist('garyfyFigure','file') && exist('myprint','file')
    % Code to clean up and save figure.
    % These functions are not in the standard Matlab release.
    figure(1)
    garyfyFigure
    myprint(1,'DCOL_NomInput')
    
    figure(2)
    garyfyFigure
    myprint(2,'DCOL_NomOutput')
end



%% Simulations with Parameter Variations
% Simulate step responses with coefficient values selected within
% the range of uncertainty. Also overlay the nominal motor response.


% Simulation and plot settings
d =  0;          % Input disturbance, V
Tf = 5;          % Final simulation time, sec
co = get(groot,'defaultAxesColorOrder');

% Plot step responses
figure(3); clf;
figure(4); clf;
figure(5); clf;
for i=1:5
    % Simulations 1-4 correspond to +/-10% uncertainty in the
    % ODE coefficients. Simulation 5 gives the nominal response.
    if i==1
        a0 = 1.1*a0Nom;
        b0 = 1.1*b0Nom;
    elseif i==2
        a0 = 1.1*a0Nom;
        b0 = 0.9*b0Nom;
    elseif i==3
        a0 = 0.9*a0Nom;
        b0 = 1.1*b0Nom;
    elseif i==4
        a0 = 0.9*a0Nom;
        b0 = 0.9*b0Nom;
    else
        % Use nominal values for last simulation
        a0 = a0Nom;
        b0 = b0Nom;
    end
    
    % Set numerator & denominator coefficients
    Gnum = b0;       % Numerator coefficients of G
    Gden = [1 a0];   % Denominator coefficients of G
        
    % Simulate Simulink model
    r = 1000;   % Desired speed, rad/sec
    sim('MotorOpenLoop')
    
    % Plot input voltage
    figure(3);
    plot(t,u,'b');
    hold on;
    
    % Plot motor speed
    figure(4);
    if i<5
        ph(i)=plot(t,y,'r-.');
    else
        ph(i)=plot(t,y,'b');
    end
    hold on;
    
    % Plot tracking error
    figure(5)
    if i<5
        plot(t,r-y,'r-.');
    else
        plot(t,r-y,'b');
    end
    hold on;
end
figure(4);
ph(6) = plot([0 Tf],[r r],'k:');

figure(3);
xlabel('Time (sec)');
ylabel('Input u (V)');
grid on;
hold off;
axis([0 Tf 0 umax]);
set(gca, 'LooseInset', get(gca, 'TightInset'));

figure(4);
xlabel('Time (sec)');
ylabel('Speed, y (rad/sec)');
legend([ph(5); ph(4); ph(6)],'Nominal','+/-10% Variation','Desired',...
    'Location','Best');
axis([0 Tf 0 1500]);
grid on;
hold off;
set(gca, 'LooseInset', get(gca, 'TightInset'));

figure(5);
xlabel('Time (sec)');
ylabel('Error, e (rad/sec)');
axis([0 Tf -300 1100]);
grid on;
hold off;
set(gca, 'LooseInset', get(gca, 'TightInset'));

if exist('garyfyFigure','file') && exist('myprint','file')
    % Code to clean up and save figure.
    % These functions are not in the standard Matlab release.
    figure(3)
    garyfyFigure
    myprint(3,'DCOL_PVInput')
    
    figure(4)
    garyfyFigure
    myprint(4,'DCOL_PVOutput')
    
    figure(5)
    garyfyFigure
    myprint(5,'DCOL_PVError')
end



