%% PID Control for Second-Order Systems


%% Second-Order Plant Model
%   d^2y/dt^2 + a1 dy/dt + a0 y = b0 u + b0 d
a1 = -2;
a0 = 17;
b0 = 17;
Gnum = b0;
Gden = [1 a1 a0];
G = tf(Gnum,Gden);

%% Desired Closed-Loop Poles
% Select closed-loop poles to achieve settling time <= 0.75 sec
% and peak overshoot Mp <= 0.05.

% Design 1: Critically Damped
zeta1 = 1;
wn1 = 5;
p1 = 5;

% Design 2: Underdamped
zeta2 = 0.7;
wn2 = 5/zeta2;
p2 = 5;

% zeta2 = 1;
% wn2 = 5;
% p2 = 5;



%% Select PID Gains
% The gains are selected to place the closed-loop poles at the
% desired locations. The controller transfer function is:
%      K(s) = (Kd*s^2+Kp*s+Ki)/s = Kd*s + Kp + Ki/s

% Design 1
Ki1 = (p1*wn1^2)/b0;
Kp1 = (2*zeta1*wn1*p1+wn1^2-a0)/b0;
Kd1 = (2*zeta1*wn1+p1-a1)/b0;
K1 = tf([Kd1 Kp1 Ki1],[1 0]);

% Design 2
Ki2 = (p2*wn2^2)/b0;
Kp2 = (2*zeta2*wn2*p2+wn2^2-a0)/b0;
Kd2 = (2*zeta2*wn2+p2-a1)/b0;
K2 = tf([Kd2 Kp2 Ki2],[1 0]);

L1 = G*K1;
T1 = feedback(L1,1);

L2 = G*K2;
T2 = feedback(L2,1);

figure(3);
bodemag(G,'k',L1,'b',L2,'r',T1,'b--',T2,'r--')

%% Simulations with Two PID Controllers
% The derivative term is implemented with an (approximate) 
% smoothed derivative.

% Simulation and plot settings
r = 4;         % Reference
d = 2;         % Disturbance
Tf = 4;         % Final simulation time, sec
LineStr = {'b','r-.'};  % Colors and line types for plot

figure(1); clf;
figure(2); clf;
for i=1:2
    if i==1
        % Design 1
        Kp = Kp1;
        Ki = Ki1;
        Kd = Kd1;
    else
        % Design 2
        Kp = Kp2;
        Ki = Ki2;
        Kd = Kd2;
    end
    
    % Simulate Simulink model
    sim('SecondOrderPIDControl')
    
    % Plot input
    figure(1);
    plot(t,u,LineStr{i});
    hold on;
    xlabel('Time (sec)');
    ylabel('Input u');
    grid on;
    xlim([0 Tf]);
    set(gca, 'LooseInset', get(gca, 'TightInset'));
    
    % Plot output
    figure(2);
    plot(t,y,LineStr{i});
    hold on;
    xlabel('Time (sec)');
    ylabel('Output y');
    xlim([0 Tf]);
    grid on;
    set(gca, 'LooseInset', get(gca, 'TightInset'));
end

figure(1)
legend('K_1(s)','K_2(s)','Location','Best');
hold off;

figure(2)
legend('K_1(s)','K_2(s)','Location','Best');
plot([0 Tf],[r r],'k:')
hold off;


if exist('garyfyFigure','file') && exist('myprint','file')
    % Code to clean up and save figure.
    % These functions are not in the standard Matlab release.
    figure(1)
    garyfyFigure
    myprint(1,'PIDSecondOrderU')
    
    figure(2)
    garyfyFigure
    myprint(2,'PIDSecondOrderY')    
end
