%% PI Control for First-Order Systems


%% First-Order Plant Model
%   dy/dt + a0 y = b0 u + b0 d
a0 = 2;
b0 = 3;
Gnum = b0;
Gden = [1 a0];
G = tf(Gnum,Gden);

%% Desired Closed-Loop Poles
% Select closed-loop poles to achieve settling time <= 0.75 sec
% and peak overshoot Mp <= 0.05.

% Desired Time Constant
% A settling time of 0.75sec corresponds to a time constant of 0.25sec.
tau = 0.25;

% Design 1: Critically Damped
zeta1 = 1;
wn1 = 1/(zeta1*tau);

% Design 2: Underdamped
zeta2 = 0.7;
wn2 = 1/(zeta2*tau);

%% Select PI Gains
% The gains are selected to place the closed-loop poles at the
% desired locations. The controller transfer function is:
%      K(s) = (Kp*s+Ki)/s = Kp + Ki/s

% Design 1
Ki1 = wn1^2/b0;
Kp1 = (2*zeta1*wn1-a0)/b0;
K1 = tf([Kp1 Ki1],[1 0]);

% Design 2
Ki2 = wn2^2/b0;
Kp2 = (2*zeta2*wn2-a0)/b0;
K2 = tf([Kp2 Ki2],[1 0]);

%% Simulations with Two PI Controllers

% Simulation and plot settings
r = 4;         % Reference
d = 2;         % Disturbance
Tf = 4;         % Final simulation time, sec
LineStr = {'b','r-.'};  % Colors and line types for plot

figure(1); clf;
figure(2); clf;
for i=1:2
    if i==1
        % Design 1
        Kp = Kp1;
        Ki = Ki1;
    else
        % Design 2
        Kp = Kp2;
        Ki = Ki2;
    end
    
    % Simulate Simulink model
    sim('FirstOrderPIControl')
    
    % Plot input
    figure(1);
    plot(t,u,LineStr{i});
    hold on;
    xlabel('Time (sec)');
    ylabel('Input u');
    grid on;
    xlim([0 Tf]);
    set(gca, 'LooseInset', get(gca, 'TightInset'));
    
    % Plot output
    figure(2);
    plot(t,y,LineStr{i});
    hold on;
    xlabel('Time (sec)');
    ylabel('Output y');
    xlim([0 Tf]);
    grid on;
    set(gca, 'LooseInset', get(gca, 'TightInset'));
end

figure(1)
legend('K_1(s)','K_2(s)','Location','Best');
hold off;

figure(2)
legend('K_1(s)','K_2(s)','Location','Best');
plot([0 Tf],[r r],'k:')
hold off;


if exist('garyfyFigure','file') && exist('myprint','file')
    % Code to clean up and save figure.
    % These functions are not in the standard Matlab release.
    figure(1)
    garyfyFigure
    myprint(1,'PIFirstOrderU')
    
    figure(2)
    garyfyFigure
    myprint(2,'PIFirstOrderY')    
end
